/*
* AD770X Library
 * Kerry D. Wong
 * http://www.kerrywong.com
 * 3/2011
 */

#include "AD770X.h"

//write communication register
//   7        6      5      4      3      2      1      0
//0/DRDY(0) RS2(0) RS1(0) RS0(0) R/W(0) STBY(0) CH1(0) CH0(0)
void AD770X::setNextOperation(byte reg, byte channel, byte readWrite)
{
  byte r = 0;
  r = reg << 4 | readWrite << 3 | channel;
  
  digitalWrite(pinCS, LOW);
  spiTransfer(r);
  digitalWrite(pinCS, HIGH);
}

//Clock Register
//   7      6       5        4        3        2      1      9
//ZERO(0) ZERO(0) ZERO(0) CLKDIS(0) CLKDIV(0) CLK(1) FS1(0) FS0(1)
//
//CLKDIS: master clock disable bit
//CLKDIV: clock divider bit
void AD770X::writeClockRegister(byte CLKDIS, byte CLKDIV, byte outputUpdateRate)
{
  byte r = CLKDIS << 4 | CLKDIV << 3 | outputUpdateRate;
  
  digitalWrite(pinCS, LOW);
  spiTransfer(r);
  digitalWrite(pinCS, HIGH);  
}

//Setup Register
//  7     6     5     4     3      2      1      0
//MD10) MD0(0) G2(0) G1(0) G0(0) B/U(0) BUF(0) FSYNC(1)
void AD770X::writeSetupRegister(byte operationMode, byte gain, byte unipolar, byte buffered, byte fsync)
{
  byte r = operationMode << 6 | gain << 3 | unipolar << 2 | buffered << 1 | fsync;
 
  digitalWrite(pinCS, LOW);
  spiTransfer(r);
  digitalWrite(pinCS, HIGH);   
}

unsigned int AD770X::readADResult()
{
   digitalWrite(pinCS, LOW);
   byte b1 = spiTransfer(0xff);
   byte b2 = spiTransfer(0xff);
   digitalWrite(pinCS, HIGH);
   
   unsigned int r = b1 << 8 | b2;
   
   return r;
}

double AD770X::readADResult(byte channel)
{
  setNextOperation(REG_DATA, channel, 1);
  return (readADResult() - 32768.0) * 1.0 / 32768.0 * VRef;
}

AD770X::AD770X(double vref)
{
    VRef = vref;
    pinMode(pinMOSI, OUTPUT);
    pinMode(pinMISO, INPUT);
    pinMode(pinSPIClock, OUTPUT);
    pinMode(pinCS, OUTPUT);
    
    digitalWrite(pinCS, HIGH);
    SPCR = _BV(SPE) | _BV(MSTR) | _BV(CPOL) | _BV(CPHA) | _BV(SPI2X) | _BV(SPR1) | _BV(SPR0);
}

void AD770X::init(byte channel, byte gain, byte updRate)
{
    setNextOperation(REG_CLOCK, channel, 0);
    writeClockRegister(0, 0, updRate);
    setNextOperation(REG_SETUP, channel, 0);
    writeSetupRegister(MODE_SELF_CAL, gain, 0, 0, 0);
}

void AD770X::init(byte channel)
{
  init(channel, GAIN_1, UPDATE_RATE_50);
}
